﻿option explicit

' "Smart" front end to Beyond Compare
'
' v2.0    (2014/04/09) simpler, cleaner rewrite
' v2.1    (2014/04/09) add user configurable path to BCompare.exe via Preferences dialog
'         (thanks to wowbagger for the suggestion)
'         use fsu.resolve consistently to fully resolve paths
' v2.2    (2014/04/10) add proper versioning and default icon
' v2.2 LD (2014/04/29) Modified version by Leo Davidson:
'                      - Requires Directory Opus 11.3.1 or above.
'                      - Ensures left and right in BeyondCompare match the left and right in Opus (even if the right is the source).
'                      - Allows comparing folders with archives (since BC supports this), with a new config option to disable it.
'                      - If one item is selected in the source and nothing (or the wrong kind of thing, or too many things) in the dest,
'                        it now ignores the destination selection(s) and compares the selected source item on both sides. For example, if
'                        C:\ and D:\ are the source/dest folders, and File.txt is selected in the source, C:\File.txt and D:\File.txt are compared.
'                        "Wrong kind of thing" means a file on one side and a folder on the other. (Where archives are considered folders.)
'                      - Fix for when the Desktop is compared to another folder. (Compares /desktopdir instead of /desktop virtual folder.)
'                      - Gets default exe path from the registry if the script config has no path or has a path that doesn't exist.
'                        Failing that, it uses the standard Beyond Compare install path, under the 32-bit Program Files folder.
'                        Default config path is now blank, and should only be set if the above methods need to be overridden.
'                      - Shows an error message if Beyond Compare exe isn't found.
'                      - If the path found/configured is for BCompare.exe, it will swap it for BComp.exe (if it exists).
'                        Using BComp.exe is preferable as it will open new tabs in an existing BC instance. (Could make this an option.)
''                       (2020: It no longer seems to matter which program is used. Both open new tabs unless "/solo" is specified.)
'                      - Won't do anything if nothing is to be compared.
'                        (If people want the button to be able to run BC on its own, it could be a config option.
'                         For me it'd just mean having to close BC if I clicked in error, so I added the check.)
'                      - Won't do anything if either thing to be compared does not exist.
'                        (This is mainly for the new case where you can select a file on one side to compare the same thing on both sides.)
'                      - New small icon matching the 22x22 size of Opus's other small toolbar icons, with 1 pixel padding.
'                        (Made by resizing the 32x32 icon to 20x20, then adding 1 pixel of padding on all sides, to match the standard icons.)
'                      - Removed debug stuff I didn't need, changed command name, and tided up the code.
' v2.2 LD2 (2020/08/24) Modified version by Leo Davidson:
'                      - Added NEWWINDOW argument (passes "/solo" switch to make BComp.exe open a new window).
'                      - Updated default/fallback path to the Beyond Compare 4 one.
' v2.2 LD3 (2023/11/16) Modified version by Leo Davidson:
'                      - Fixed problem with files that have archive extensions handled by plugins, but aren't valid archives.

Function OnInit(ByRef initData) ' OnInit is called by DOpus to initialize the script
   
	initData.name = "Smart Beyond Compare"
	initData.version= "2.2 LD3"
	initData.desc = "Run Beyond Compare on selected files and folders."
	initData.copyright = "(c) 2014 AussieBoykie (mod by Leo Davidson)"
	initData.url = "https://resource.dopus.com/t/smart-front-end-for-beyond-compare/17115/46?u=leo"
	initData.default_enable = true
	initData.min_version = "11.4"

	initData.config.BCPath = ""
	initData.config.ArchivesAsFolders = True

	dim cmd : set cmd = initData.AddCommand()
	cmd.name = "BeyondCompare" ' This is the name of the command being added
	cmd.method = "Main" ' This is the routine to execute when the BeyondCompare command is invoked 
	cmd.desc = initData.desc
	cmd.label = "Compare"
	cmd.icon = "BeyondCompare:BeyondCompare"
	cmd.template = "NEWWINDOW/S"

End Function

Function Main(ByRef scriptCmdData)

	Dim cdf : Set cdf = scriptCmdData.func
	Dim fsu : Set fsu = DOpus.FSUtil

	Dim tempPath

	' Prevent deselection. This this at the start so it's done even if we exit early.   
	cdf.command.deselect = false

	Dim getTypeMode : getTypeMode = ""
	If (Not Script.config.ArchivesAsFolders) Then
		getTypeMode = "a" ' GetType "a" flag means treat archives as files, not folders.
	End If

	' ********************************
	' *** Get path to BCompare.exe ***
	' ********************************

	Dim BCPath : BCPath = Script.config.BCPath

	If (BCPath = "") Or (Not fsu.Exists(BCPath)) Then
		Const HKEY_CLASSES_ROOT   = &H80000000
		Const HKEY_CURRENT_USER   = &H80000001
		Const HKEY_LOCAL_MACHINE  = &H80000002
		Const HKEY_USERS          = &H80000003
		Dim reg : Set reg = GetObject("winmgmts:{impersonationLevel=impersonate}!\\.\root\default:StdRegProv")
		Dim strValue
		reg.GetStringValue HKEY_CURRENT_USER, "Software\Scooter Software\Beyond Compare", "ExePath", strValue
		If (Not IsNull(strValue)) Then
	        	BCPath = strValue
		Else
			BCPath = fsu.resolve("/programfiles\Beyond Compare 4\BCompare.exe")
		End If
	End If

	If (BCPath = "") Or (Not fsu.Exists(BCPath)) Then
		cdf.Dlg.Request "BCompare.exe could not be found automatically. You can configure the script with an explicit path if needed.", "OK", "Beyond Compare script"
		Exit Function
	End If

	' ******************************************************
	' *** Change BCompare.exe to BComp.exe, if available ***
	' ******************************************************

	Set tempPath = fsu.NewPath(BCPath)
	If tempPath.filepart = "BCompare.exe" Then
		tempPath.Parent
		tempPath.Add "BComp.exe"
		If fsu.Exists(tempPath) Then
			BCPath = tempPath
		End If
	End If
	Set tempPath = Nothing
   
	' ****************************
	' *** Get paths to compare ***
	' ****************************

	Dim leftPath  : Set leftPath = Nothing
	Dim rightPath : Set rightPath = Nothing

	' We compare types using fsu.GetType, rather than using selected_dirs and selected_files,
	' because we want the feature of fsu.GetType where it (by default) considers archives
	' as folders. This is because Beyond Compare can do the same and it allows people to
	' e.g. compare archive backups against the folders they were made from.
	' The user can turn this mode off, which affects the getTypeMode variable.

	If cdf.sourcetab.selected.count = 2 Then
		' Two items selected in SOURCE
		' (Types will be checked later)
		Set leftPath = fsu.NewPath(cdf.sourcetab.selected(0))
		Set rightPath = fsu.NewPath(cdf.sourcetab.selected(1))
	ElseIf cdf.desttab <> 0 Then

		If cdf.sourcetab.selected.count = 1 And cdf.desttab.selected.count = 1 Then
			' Separate if-statement because VBScript doesn't have short-circuit logic (SIGH!)
			' and it's not safe to access selected(0) until we know the counts are non-zero.
			If fsu.GetType(cdf.sourcetab.selected(0), getTypeMode) = fsu.GetType(cdf.desttab.selected(0), getTypeMode) Then
				' A single item of the same type selected in both SOURCE and DEST
				Set leftPath = fsu.NewPath(cdf.sourcetab.selected(0))
				Set rightPath = fsu.NewPath(cdf.desttab.selected(0))
			End If
		End If
		' Try other cases if the inner if-statement above did not hold.
		If (leftPath Is Nothing) Or (rightPath Is Nothing) Then
			If cdf.sourcetab.selected.count = 1 Then
				' One item selected in SOURCE, and ignore whatever is selected in DEST
				' Compare against the same item in both sides
				' (Types will be checked later)
				Set leftPath = fsu.NewPath(cdf.sourcetab.selected(0))
				Set rightPath = fsu.NewPath(cdf.desttab.path)
				rightPath.Add(leftPath.filepart)
			Else
				' SOURCE and DEST folders will be compared
				Set leftPath = fsu.NewPath(cdf.sourcetab.path)
				Set rightPath = fsu.NewPath(cdf.desttab.path)
			End If
		End If
	End If

	' **************************
	' *** Build command line ***
	' **************************

	' Basic BC command with no arguments
	Dim cmdline : cmdline = """" & BCPath & """"

	If (leftPath Is Nothing) Or (rightPath Is Nothing) Then
		' If nothing is to be compared, don't run BC on its own.
		' (Could make this a config option if people want the button to be able to launch BC by itself.)
		Exit Function   
	Else

		If scriptCmdData.func.args.got_arg.newwindow Then
			cmdline = cmdline & " /solo"
		End If

		If cdf.command.IsSet("SOURCE=Right") Then
			' The paths are the wrong way around, so swap them.
			Set tempPath = leftPath
			Set leftPath = rightPath
			Set rightPath = tempPath
			Set tempPath = Nothing
		End If

		' Use DisplayName "er" instead of Resolve, as we need to not only resolve the paths
		' but also convert virtual folder paths, like Desktop, into normal paths.
		leftPath = fsu.DisplayName(leftPath, "er")
		rightPath = fsu.DisplayName(rightPath, "er")

		Dim leftType, rightType
		leftType  = fsu.GetType(leftPath, getTypeMode)
		rightType = fsu.GetType(rightPath, getTypeMode)
		
		If leftType = "invalid" Or rightType = "invalid" Then
			Exit Function
		End If

		' Test that the two things being compared are of the same file/dir type.
		If leftType <> rightType Then
			Exit Function
		End If

		cmdline = cmdline & " " & """" & leftPath & """" & " " & """" & rightPath & """"
	End If
   
	' The REPLACE function in the following statement appears to be needed
	' to prevent the possibility of (e.g.) %1, %2, etc. being misinterpreted

	cmdline = Replace(cmdline, "%", "%%") ' Execute Beyond Compare with appropriate args 

	' ***********************
	' *** Run the command ***
	' ***********************

	cdf.command.runcommand cmdline
   
End Function
